==> steam-tools-ng-1.1.1.tar.gz: steam-tools-ng-1.1.1/src/steam_tools_ng/config.py <==
#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import configparser
import http
import locale
import logging
import os
import sys
import time
from collections import OrderedDict
from pathlib import Path
from typing import Any

import aiohttp
from stlib import plugins as stlib_plugins
from stlib import webapi

from . import i18n, logger_handlers

parser = configparser.RawConfigParser()
log = logging.getLogger(__name__)
_ = i18n.get_translation

if Path('src').is_dir():
    # development mode
    data_dir = Path('config')
elif hasattr(sys, 'frozen') or sys.platform == 'win32':
    data_dir = Path(os.environ['LOCALAPPDATA'])
else:
    data_dir = Path(os.getenv('XDG_CONFIG_HOME', Path.home() / '.config'))

config_file_directory = data_dir / 'steam-tools-ng'
config_file_name = 'steam-tools-ng.config'
config_file = config_file_directory / config_file_name

try:
    from stlib import client
except ImportError as exception:
    log.error(str(exception))
    client = None

gtk_themes = OrderedDict([
    ('light', "Light"),
    ('dark', "Dark"),
])

log_levels = OrderedDict([
    ('critical', "Critical"),
    ('error', "Error"),
    ('warning', "Warning"),
    ('info', "Info"),
    ('debug', "Debug"),
])

translations = OrderedDict([
    ('en', "English"),
    ('pt_BR', "Portuguese (Brazil)"),
    ('fr', "French"),
])

giveaway_types = OrderedDict([
    ('main', "Main Giveaways"),
    ('new', "New Giveaways"),
    ('recommended', "Recommended"),
    ('wishlist', "Wishlist Only"),
    ('group', 'Group Only'),
])

giveaway_sort_types = OrderedDict([
    ('name', "Name"),
    ('copies', "Copies"),
    ('points', "Points"),
    ('level', "Level"),
])

plugins = OrderedDict([
    ("confirmations", "Confirmations"),
    ("steamtrades", "Steam Trades"),
    ("steamgifts", "Steam Gifts"),
    ("steamguard", "Steam Guard"),
    ("cardfarming", "Card Farming"),
])

if sys.platform == 'win32':
    event_loop = asyncio.ProactorEventLoop()
    file_manager = 'explorer'
else:
    file_manager = 'xdg-open'
    event_loop = asyncio.new_event_loop()

asyncio.set_event_loop(event_loop)

default_config = {
    'logger': {
        'log_directory': data_dir / 'steam-tools-ng',
        'log_level': 'debug',
        'log_console_level': 'info',
        'log_color': True,
    },
    'steam': {
        'api_url': 'https://api.steampowered.com',
        'api_key': '',
    },
    'confirmations': {
        'enable': True,
    },
    'steamguard': {
        'enable': True,
    },
    'steamtrades': {
        'enable': True,
        'wait_min': 3700,
        'wait_max': 4100,
        'trade_ids': '',
    },
    'steamgifts': {
        'enable': True,
        'wait_min': 3700,
        'wait_max': 4100,
        'giveaway_type': 'main',
        'developer_giveaways': 'True',
        'sort': 'name',
        'reverse_sorting': False,
    },
    'cardfarming': {
        'enable': True,
        'reverse_sorting': False,
        'first_wait': 7200,
        'default_wait': 600,
        'min_wait': 120,
    },
    'general': {
        'theme': 'light',
        'show_close_button': True,
        'language': str(locale.getdefaultlocale()[0]),
    },
    'login': {
        'steamid': 0,
        'deviceid': '',
        'token': '',
        'token_secure': '',
        'oauth_token': '',
        'account_name': '',
        'shared_secret': '',
        'identity_secret': '',
        'password': '',
    },
}


def update_log_level(type_: str, level_string: str) -> None:
    level = getattr(logging, level_string.upper())
    file_handler, console_handler = logging.root.handlers

    if type_ == "console":
        console_handler.setLevel(level)
    else:
        file_handler.setLevel(level)


def validate_config(section: str, option: str, defaults: OrderedDict) -> None:
    value = parser.get(section, option)

    if value and value not in defaults.keys():
        if option == 'language':
            log.error(_("Unsupported language requested. Fallbacking to English."))
            new('general', 'language', 'en')
            return

        raise configparser.Error(_("Please, fix your config file. Available values for {}:\n{}").format(
            option,
            ', '.join(defaults.keys()),
        ))


def init() -> None:
    config_file_directory.mkdir(parents=True, exist_ok=True)
    parser.read_dict(default_config)

    if config_file.is_file():
        parser.read(config_file)

    # fallback deprecated values
    if (parser.get('steam', 'api_url') in [
        'https://api.lara.monster', 'https://api.lara.click',
    ]):
        new('steam', 'api_url', default_config['steam']['api_url'])

    log_directory = Path(parser.get("logger", "log_directory"))

    if not log_directory.is_dir():
        log.error(_("Incorrect log directory. Fallbacking to default."))
        log_directory = data_dir / 'steam-tools-ng'
        new("logger", "log_directory", log_directory)

    validate_config("logger", "log_level", log_levels)
    validate_config("logger", "log_console_level", log_levels)
    validate_config("general", "theme", gtk_themes)
    validate_config("general", "language", translations)
    validate_config("steamgifts", "giveaway_type", giveaway_types)
    validate_config("steamgifts", "sort", giveaway_sort_types)

    log_directory.mkdir(parents=True, exist_ok=True)

    if not stlib_plugins.has_plugin("steamtrades"):
        new("plugins", "steamtrades", False)

    if not stlib_plugins.has_plugin("steamgifts"):
        new("plugins", "steamgifts", False)

    if not client:
        new("plugins", "cardfarming", False)


def init_logger() -> None:
    log_directory = Path(parser.get("logger", "log_directory"))
    log_level = parser.get("logger", "log_level")
    log_console_level = parser.get("logger", "log_console_level")

    log_file_handler = logger_handlers.RotatingFileHandler(log_directory / 'steam-tools-ng.log',
                                                           backupCount=1,
                                                           encoding='utf-8')
    log_file_handler.setFormatter(logging.Formatter('%(module)s:%(levelname)s (%(funcName)s) => %(message)s'))
    log_file_handler.setLevel(getattr(logging, log_level.upper()))

    try:
        log_file_handler.doRollover()
    except PermissionError:
        log.debug(_("Unable to open steam-tools-ng.log"))
        log_file_handler.close()
        log_file_handler = logger_handlers.NullHandler()  # type: ignore

    log_console_handler = logger_handlers.ColoredStreamHandler()
    log_console_handler.setLevel(getattr(logging, log_console_level.upper()))

    # noinspection PyArgumentList
    logging.basicConfig(level=logging.DEBUG, handlers=[log_file_handler, log_console_handler])


def new(section: str, option: str, value: Any) -> None:
    if option == "log_level":
        update_log_level("file", value)
    elif option == "log_console_level":
        update_log_level("console", value)

    if parser.get(section, option, fallback='') != str(value):
        log.debug(_('Saving {}:{} on config file').format(section, option))
        parser.set(section, option, str(value))

        with open(config_file, 'w', encoding="utf8") as config_file_object:
            parser.write(config_file_object)
    else:
        log.debug(_('Not saving {}:{} because values are already updated').format(section, option))


def remove(section: str, option: str) -> None:
    # Some GUI checks will fail if option doesn't exists
    new(section, option, '')
    # parser.remove_option(section, option)

    # with open(config_file, 'w', encoding="utf8") as config_file_object:
    #    parser.write(config_file_object)


def login_cookies() -> http.cookies.SimpleCookie:
    steamid = parser.getint("login", "steamid")
    token = parser.get("login", "token")
    token_secure = parser.get("login", "token_secure")

    if not steamid or not token or not token_secure:
        log.warning(_("No login cookies"))
        return {}

    cookies_dict = {
        'steamLogin': f'{steamid}%7C%7C{token}',
        'steamLoginSecure': f'{steamid}%7C%7C{token_secure}',
    }

    return http.cookies.SimpleCookie(cookies_dict)


async def time_offset(webapi_session: webapi.SteamWebAPI) -> int:
    try:
        if not client:
            raise ProcessLookupError

        with client.SteamGameServer() as server:
            server_time = server.get_server_time()
    except ProcessLookupError:
        log.warning(_("Steam is not running."))
        log.debug(_("Fallbacking time offset to WebAPI"))

        while True:
            try:
                server_time = await webapi_session.get_server_time()
            except aiohttp.ClientError:
                raise aiohttp.ClientError(
                    _(
                        "Unable to Connect. You can try these things:\n"
                        "1. Check your connection\n"
                        "2. Check if Steam Server isn't down\n"
                        "3. Check if api_url and api_key is correct on config file\n"
                    )
                )
            else:
                break

    return int(time.time()) - server_time

==> steam-tools-ng-2.0-3.tar.gz: steam-tools-ng-2.0/src/steam_tools_ng/config.py <==
#!/usr/bin/env python
#
# Lara Maia <dev@lara.monster> 2015 ~ 2022
#
# The Steam Tools NG is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# The Steam Tools NG is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see http://www.gnu.org/licenses/.
#
import asyncio
import configparser
import locale
import logging
import os
import site
import sys
from collections import OrderedDict
from pathlib import Path
from typing import Any, Mapping

from stlib import plugins as stlib_plugins
from . import i18n, logger_handlers

parser = configparser.RawConfigParser()
log = logging.getLogger(__name__)
_ = i18n.get_translation

if Path('src').is_dir():
    # development mode
    data_dir = Path('config')
elif hasattr(sys, 'frozen') or sys.platform == 'win32':
    data_dir = Path(os.environ['LOCALAPPDATA'])
else:
    data_dir = Path(os.getenv('XDG_CONFIG_HOME', Path.home() / '.config'))

config_file_directory = data_dir / 'steam-tools-ng'
config_file_name = 'steam-tools-ng.config'
config_file = config_file_directory / config_file_name

try:
    from stlib import client
except ImportError as exception:
    log.error(str(exception))
    client = None

gtk_themes = OrderedDict([
    ('light', "Light"),
    ('dark', "Dark"),
])

log_levels = OrderedDict([
    ('critical', "Critical"),
    ('error', "Error"),
    ('warning', "Warning"),
    ('info', "Info"),
    ('debug', "Debug"),
])

translations = OrderedDict([
    ('en', "English"),
    ('pt_BR', "Portuguese (Brazil)"),
    ('fr', "French"),
])

giveaway_types = OrderedDict([
    ('main', "Main Giveaways"),
    ('new', "New Giveaways"),
    ('recommended', "Recommended"),
    ('wishlist', "Wishlist Only"),
    ('group', 'Group Only'),
])

giveaway_sort_types = OrderedDict([
    ('name', "Name"),
    ('copies', "Copies"),
    ('points', "Points"),
    ('level', "Level"),
])

plugins = OrderedDict([
    ("coupons", "Free Coupons"),
    ("confirmations", "Confirmations"),
    ("steamtrades", "Steam Trades"),
    ("steamgifts", "Steam Gifts"),
    ("steamguard", "Steam Guard"),
    ("cardfarming", "Card Farming"),
])

if sys.platform == 'win32':
    event_loop = asyncio.ProactorEventLoop()
    file_manager = 'explorer'
else:
    file_manager = 'xdg-open'
    event_loop = asyncio.new_event_loop()

asyncio.set_event_loop(event_loop)

default_config: Mapping[str, Mapping[str, Any]] = {
    'logger': {
        'log_directory': data_dir / 'steam-tools-ng',
        'log_level': 'debug',
        'log_console_level': 'info',
        'log_color': True,
    },
    'steam': {
        'api_url': 'https://api.steampowered.com',
    },
    'coupons': {
        'enable': True,
        'botid_to_donate': '76561198018370992',
        'botids': '76561198018370992',
        'appid': '753',
        'contextid': '3',
        'token_to_donate': '6Z6Xn5NM',
        'tokens': '6Z6Xn5NM',
        'last_trade_time': 0,
    },
    'confirmations': {
        'enable': True,
    },
    'steamguard': {
        'enable': True,
    },
    'steamtrades': {
        'enable': True,
        'wait_min': 3700,
        'wait_max': 4100,
        'trade_ids': '',
    },
    'steamgifts': {
        'enable': True,
        'wait_min': 3700,
        'wait_max': 4100,
        'giveaway_type': 'main',
        'developer_giveaways': 'True',
        'sort': 'name',
        'reverse_sorting': False,
    },
    'cardfarming': {
        'enable': True,
        'reverse_sorting': False,
        'first_wait': 7200,
        'default_wait': 600,
        'min_wait': 120,
        'max_concurrency': 5,
        'invisible': True,
    },
    'general': {
        'theme': 'light',
        'show_close_button': True,
        'language': str(locale.getdefaultlocale()[0]),
    },
    'login': {
        'steamid': 0,
        'deviceid': '',
        'token': '',
        'token_secure': '',
        'oauth_token': '',
        'account_name': '',
        'shared_secret': '',
        'identity_secret': '',
        'password': '',
    },
}


def update_log_level(type_: str, level_string: str) -> None:
    level = getattr(logging, level_string.upper())
    file_handler, console_handler, *extra_handlers = logging.root.handlers

    if type_ == "console":
        console_handler.setLevel(level)
    else:
        file_handler.setLevel(level)


def validate_config(section: str, option: str, defaults: OrderedDict[str, str]) -> None:
    value = parser.get(section, option)

    if value and value not in defaults.keys():
        if option == 'language':
            log.error(_("Unsupported language requested. Fallbacking to English."))
            new('general', 'language', 'en')
            return

        raise configparser.Error(_("Please, fix your config file. Available values for {}:\n{}").format(
            option,
            ', '.join(defaults.keys()),
        ))


def init() -> None:
    config_file_directory.mkdir(parents=True, exist_ok=True)
    parser.read_dict(default_config)

    if config_file.is_file():
        parser.read(config_file)

    # fallback deprecated values
    if (parser.get('steam', 'api_url') in [
        'https://api.lara.monster', 'https://api.lara.click',
    ]):
        new('steam', 'api_url', default_config['steam']['api_url'])

    log_directory = Path(parser.get("logger", "log_directory"))

    if not log_directory.is_dir():
        log.error(_("Incorrect log directory. Fallbacking to default."))
        log_directory = data_dir / 'steam-tools-ng'
        new("logger", "log_directory", log_directory)

    validate_config("logger", "log_level", log_levels)
    validate_config("logger", "log_console_level", log_levels)
    validate_config("general", "theme", gtk_themes)
    validate_config("general", "language", translations)
    validate_config("steamgifts", "giveaway_type", giveaway_types)
    validate_config("steamgifts", "sort", giveaway_sort_types)

    log_directory.mkdir(parents=True, exist_ok=True)

    stlib_plugins.add_search_paths(
        str(Path(os.getcwd(), 'lib', 'stlib-plugins')),
        *[str(Path(site_, 'stlib-plugins')) for site_ in site.getsitepackages()],
        str(Path(site.getusersitepackages(), 'stlib-plugins')),
    )

    if not stlib_plugins.has_plugin("steamtrades"):
        new("steamtrades", "enable", False)

    if not stlib_plugins.has_plugin("steamgifts"):
        new("steamgifts", "enable", False)

    if not client:
        new("cardfarming", "enable", False)


def init_logger() -> None:
    log_directory = Path(parser.get("logger", "log_directory"))
    log_level = parser.get("logger", "log_level")
    log_console_level = parser.get("logger", "log_console_level")

    log_file_handler = logger_handlers.RotatingFileHandler(log_directory / 'steam-tools-ng.log',
                                                           backupCount=1,
                                                           encoding='utf-8')
    log_file_handler.setFormatter(logging.Formatter('%(module)s:%(levelname)s (%(funcName)s) => %(message)s'))
    log_file_handler.setLevel(getattr(logging, log_level.upper()))

    try:
        log_file_handler.doRollover()
    except PermissionError:
        log.debug(_("Unable to open steam-tools-ng.log"))
        log_file_handler.close()
        log_file_handler = logger_handlers.NullHandler()  # type: ignore

    log_console_handler = logger_handlers.ColoredStreamHandler()
    log_console_handler.setLevel(getattr(logging, log_console_level.upper()))

    # noinspection PyArgumentList
    logging.basicConfig(level=logging.DEBUG, handlers=[log_file_handler, log_console_handler])


def new(section: str, option: str, value: Any) -> None:
    if option == "log_level":
        update_log_level("file", value)
    elif option == "log_console_level":
        update_log_level("console", value)

    if parser.get(section, option, fallback='') != str(value):
        log.debug(_('Saving {}:{} on config file').format(section, option))
        parser.set(section, option, str(value))

        with open(config_file, 'w', encoding="utf8") as config_file_object:
            parser.write(config_file_object)
    else:
        log.debug(_('Not saving {}:{} because values are already updated').format(section, option))


def remove(section: str, option: str) -> None:
    # Some GUI checks will fail if option doesn't exists
    new(section, option, '')
    # parser.remove_option(section, option)

    # with open(config_file, 'w', encoding="utf8") as config_file_object:
    #    parser.write(config_file_object)
